import { Event, StateValue, Transition, Action, StandardMachine, ParallelMachine, StateOrMachineConfig, MachineConfig, ParallelMachineConfig, EventType } from './types';
import matchesState from './matchesState';
import mapState from './mapState';
import State from './State';
declare class StateNode<TStateKey extends string = string, TEventType extends string = string> {
    config: StateOrMachineConfig<TStateKey, TEventType>;
    key: string;
    id: string;
    relativeId: string;
    initial?: string;
    parallel?: boolean;
    states: Record<TStateKey, StateNode>;
    on?: Record<TEventType, Transition<TStateKey> | undefined>;
    onEntry?: Action[];
    onExit?: Action[];
    strict: boolean;
    parent?: StateNode;
    machine: StateNode;
    private __cache;
    constructor(config: StateOrMachineConfig<TStateKey, TEventType>);
    getStateNodes(state: StateValue | State): StateNode[];
    handles(event: Event): boolean;
    transition(state: StateValue | State, event: Event, extendedState?: any): State;
    private transitionStateValue(state, event, extendedState?);
    private next(event, history?, extendedState?);
    private readonly initialStateValue;
    readonly initialState: State;
    getStates(stateValue: StateValue): StateNode[];
    getState(relativeStateId: string | string[]): StateNode | undefined;
    readonly events: EventType[];
    private getRelativeValue(toNode?);
    private getActions(prevStateValue, nextStateValue, actions, event);
}
export interface MachineFactory {
    (config: MachineConfig | ParallelMachineConfig): StandardMachine | ParallelMachine;
    standard: (config: MachineConfig) => StandardMachine;
    parallel: (config: ParallelMachineConfig) => ParallelMachine;
}
export declare function Machine(config: MachineConfig | ParallelMachineConfig): StandardMachine | ParallelMachine;
export { StateNode, State, matchesState, mapState };
